"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.pageDefaults = void 0;
var lodash_includes_1 = __importDefault(require("lodash.includes"));
var component_url_1 = __importDefault(require("component-url"));
/**
 * Return the canonical path for the page.
 */
var canonicalPath = function () {
    var canon = document.querySelector("link[rel='canonical']");
    if (!canon)
        return window.location.pathname;
    var href = canon.getAttribute("href");
    var parsed = component_url_1.default.parse(href);
    return parsed.pathname;
};
/**
 * Return the canonical URL for the page concat the given `search`
 * and strip the hash.
 */
var canonicalUrl = function (search) {
    var canon = document.querySelector("link[rel='canonical']");
    if (canon) {
        var href = canon.getAttribute("href");
        return lodash_includes_1.default(href, '?') ? href : href + search;
    }
    var url = window.location.href;
    var i = url.indexOf('#');
    return i === -1 ? url : url.slice(0, i);
};
/**
 * Return a default `options.context.page` object.
 *
 * https://segment.com/docs/spec/page/#properties
 */
exports.pageDefaults = function () {
    var path = canonicalPath();
    var referrer = document.referrer, title = document.title;
    var search = location.search;
    var url = canonicalUrl(search);
    return {
        path: path,
        referrer: referrer,
        search: search,
        title: title,
        url: url
    };
};
